"""Tests for the `model_cache` module."""


# TODO
class TestModelCache:
    """Test that the `ModelCache` class works as expected."""

    def test_initialisation(self) -> None:
        """Test that the `ModelCache` class can be instantiated."""
        pass

    def test_save_and_load(self) -> None:
        """Test that the `ModelCache` class can be saved and loaded."""
        pass

    def test_getitem(self) -> None:
        """Test that the `ModelCache` class can be accessed like a dictionary."""
        pass

    def test_setitem(self) -> None:
        """Test that the `ModelCache` class can be set like a dictionary."""
        pass

    def test_remove(self) -> None:
        """Test that the `ModelCache` class can be removed."""
        pass

    def test_cached_texts(self) -> None:
        """Test that the `ModelCache` class can return all the cached texts."""
        pass

    def test_add_to_cache(self) -> None:
        """Test that the `ModelCache` class can add items to the cache."""
        pass


# TODO
def test_split_dataset_into_cached_and_non_cached() -> None:
    """Test that the `split_dataset_into_cached_and_non_cached` function works."""
    pass


# TODO
def test_load_cached_model_outputs() -> None:
    """Test that the `load_cached_model_outputs` function works."""
    pass
